/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Axis" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package javax.xml.rpc.encoding;

import javax.xml.rpc.JAXRPCException;
import java.util.Iterator;

/**
 * The interface javax.xml.rpc.encoding.TypeMappingRegistry defines a registry for TypeMapping instances for 
 * the different encoding styles. 
 *
 * @version 0.1
 */
public interface TypeMappingRegistry extends java.io.Serializable {

    /**
     * The method register adds a TypeMapping instance for a specific encoding style or XML schema namespace to the 
     * type mapping registry.
     *
     * @param mapping - TypeMapping for a specific encoding style or XML schema namespace
     * @param namespaceURI - Encoding style or XML schema namespace specified as an URI. An example is 
     * "http://schemas.xmlsoap.org/soap/encoding/"
     *
     * @throws JAXRPCException - If there is any error in the registration of the TypeMapping for the specified namespace URI
     * java.lang.IllegalArgumentException - if an invalid namespace URI is specified
     */
    public void registry(TypeMapping mapping, String namespaceURI)
        throws JAXRPCException;

    /**
     * Gets a list of all TypeMapping objects registered with this TypeMappingRegistry.
     *
     * @return java.util.Iterator for the list of registered TypeMapping objects
     */
    public Iterator getTypeMappings();

    /**
     * Gets a list of namespace URIs registered with this TypeMappingRegistry.
     *
     * @return java.util.Iterator for the list of all registered namespace URIs
     */
    public Iterator getEncodingStyle();

    /**
     * Returns the registered TypeMapping for the specified encoding style or XML schema namespace.
     *
     * @param namespaceURI - Encoding style or XML schema namespace specified as an URI. An example is 
     * "http://schemas.xmlsoap.org/soap/encoding/"
     *
     * @return The registered TypeMapping for the specified encoding style or XML schema namespace. If there is no 
     * registered TypeMapping, this method returns null.
     */
    public TypeMapping getTypeMapping(String namespaceURI);

    /**
     * Creates a new empty TypeMapping object for the specified encoding style or XML schema namespace.
     *
     * @param namespaceURI - Encoding style or XML schema namespace specified as an URI. An example is 
     * "http://schemas.xmlsoap.org/soap/encoding/"
     *
     * @return An empty TypeMapping object
     */
    public TypeMapping createTypeMapping(String namespaceURI);
}


